@@ -133,11 +133,14 @@ struct TELEPATHY_QT4_NO_EXPORT Connection::Private
     ContactManagerPtr contactManager;
 
     // FeatureSelfContact
+    bool introspectingSelfContact;
+    bool reintrospectSelfContactRequired;
     ContactPtr selfContact;
     QStringList contactAttributeInterfaces;
 
     // FeatureSimplePresence
     SimpleStatusSpecMap simplePresenceStatuses;
+    uint maxPresenceStatusMessageLength;
 
     // FeatureAccountBalance
     CurrencyAmount accountBalance;
@@ -209,8 +212,14 @@ Connection::Private::Private(Connection *parent,
       selfHandle(0),
       immortalHandles(false),
       contactManager(ContactManagerPtr(new ContactManager(parent))),
+      introspectingSelfContact(false),
+      reintrospectSelfContactRequired(false),
+      maxPresenceStatusMessageLength(0),
       handleContext(0)
 {
+    accountBalance.amount = 0;
+    accountBalance.scale = 0;
+
     Q_ASSERT(properties != 0);
 
     if (chanFactory->dbusConnection().name() != parent->dbusConnection().name()) {
@@ -466,6 +475,12 @@ void Connection::Private::introspectContactAttributeInterfaces()
 void Connection::Private::introspectSelfContact(Connection::Private *self)
 {
     debug() << "Building self contact";
+
+    Q_ASSERT(!self->introspectingSelfContact);
+
+    self->introspectingSelfContact = true;
+    self->reintrospectSelfContactRequired = false;
+
     PendingContacts *contacts = self->contactManager->contactsForHandles(
             UIntList() << self->selfHandle);
     self->parent->connect(contacts,
@@ -480,9 +495,8 @@ void Connection::Private::introspectSimplePresence(Connection::Private *self)
     debug() << "Calling Properties::Get("
         "Connection.I.SimplePresence.Statuses)";
     QDBusPendingCall call =
-        self->properties->Get(
-                QLatin1String(TELEPATHY_INTERFACE_CONNECTION_INTERFACE_SIMPLE_PRESENCE),
-                QLatin1String("Statuses"));
+        self->properties->GetAll(
+                QLatin1String(TELEPATHY_INTERFACE_CONNECTION_INTERFACE_SIMPLE_PRESENCE));
     QDBusPendingCallWatcher *watcher =
         new QDBusPendingCallWatcher(call, self->parent);
     self->parent->connect(watcher,
@@ -1246,21 +1260,52 @@ uint Connection::selfHandle() const
 SimpleStatusSpecMap ConnectionLowlevel::allowedPresenceStatuses() const
 {
     if (!isValid()) {
-        warning() << "ConnectionLowlevel::selfHandle() called for a connection which is already destroyed";
+        warning() << "ConnectionLowlevel::selfHandle() "
+            "called for a connection which is already destroyed";
         return SimpleStatusSpecMap();
     }
 
     ConnectionPtr conn(mPriv->conn);
 
-    if (!conn->isReady(Features() << Connection::FeatureSimplePresence)) {
-        warning() << "Trying to retrieve simple presence from connection, but "
+    if (!conn->isReady(Connection::FeatureSimplePresence)) {
+        warning() << "Trying to retrieve allowed presence statuses from connection, but "
                      "simple presence is not supported or was not requested. "
-                     "Use becomeReady(FeatureSimplePresence)";
+                     "Enable FeatureSimplePresence in this connection";
     }
 
     return conn->mPriv->simplePresenceStatuses;
 }
 
+/**
+ * Returns the maximum length for a presence status message.
+ *
+ * The value may have changed arbitrarily during the time the
+ * Connection spends in status ConnectionStatusConnecting,
+ * again staying fixed for the entire time in ConnectionStatusConnected.
+ *
+ * This method requires Connection::FeatureSimplePresence to be enabled.
+ *
+ * \return The maximum length for a presence status message, or 0 if there is no limit.
+ */
+uint ConnectionLowlevel::maxPresenceStatusMessageLength() const
+{
+    if (!isValid()) {
+        warning() << "ConnectionLowlevel::maxPresenceStatusMessageLength() "
+            "called for a connection which is already destroyed";
+        return 0;
+    }
+
+    ConnectionPtr conn(mPriv->conn);
+
+    if (!conn->isReady(Connection::FeatureSimplePresence)) {
+        warning() << "Trying to retrieve max presence status message length connection, but "
+                     "simple presence is not supported or was not requested. "
+                     "Enable FeatureSimplePresence in this connection";
+    }
+
+    return conn->mPriv->maxPresenceStatusMessageLength;
+}
+
 /**
  * Set the self presence status.
  *
@@ -1603,11 +1648,20 @@ void Connection::gotContactAttributeInterfaces(QDBusPendingCallWatcher *watcher)
 
 void Connection::gotSimpleStatuses(QDBusPendingCallWatcher *watcher)
 {
-    QDBusPendingReply<QDBusVariant> reply = *watcher;
+    QDBusPendingReply<QVariantMap> reply = *watcher;
 
     if (!reply.isError()) {
-        mPriv->simplePresenceStatuses = qdbus_cast<SimpleStatusSpecMap>(reply.value().variant());
-        debug() << "Got" << mPriv->simplePresenceStatuses.size() << "simple presence statuses";
+        QVariantMap props = reply.value();
+
+        mPriv->simplePresenceStatuses = qdbus_cast<SimpleStatusSpecMap>(
+                props[QLatin1String("Statuses")]);
+        mPriv->maxPresenceStatusMessageLength = qdbus_cast<uint>(
+                props[QLatin1String("MaximumStatusMessageLength")]);
+
+        debug() << "Got" << mPriv->simplePresenceStatuses.size() <<
+            "simple presence statuses - max status message length is" <<
+            mPriv->maxPresenceStatusMessageLength;
+
         mPriv->readinessHelper->setIntrospectCompleted(FeatureSimplePresence, true);
     }
     else {
@@ -1630,8 +1684,7 @@ void Connection::gotSelfContact(PendingOperation *op)
         if (mPriv->selfContact != contact) {
             mPriv->selfContact = contact;
 
-            // first time
-            if (!mPriv->readinessHelper->actualFeatures().contains(FeatureSelfContact)) {
+            if (!isReady(FeatureSelfContact)) {
                 mPriv->readinessHelper->setIntrospectCompleted(FeatureSelfContact, true);
             }
 
@@ -1643,10 +1696,21 @@ void Connection::gotSelfContact(PendingOperation *op)
 
         // check if the feature is already there, and for some reason introspectSelfContact
         // failed when called the second time
-        if (!mPriv->readinessHelper->missingFeatures().contains(FeatureSelfContact)) {
+        if (!isReady(FeatureSelfContact)) {
             mPriv->readinessHelper->setIntrospectCompleted(FeatureSelfContact, false,
                     op->errorName(), op->errorMessage());
         }
+
+        if (mPriv->selfContact) {
+            mPriv->selfContact.reset();
+            emit selfContactChanged();
+        }
+    }
+
+    mPriv->introspectingSelfContact = false;
+
+    if (mPriv->reintrospectSelfContactRequired) {
+        mPriv->introspectSelfContact(mPriv);
     }
 }
 
@@ -1710,11 +1774,20 @@ Client::ConnectionInterface *Connection::baseInterface() const
     return mPriv->baseInterface;
 }
 
+/**
+ * Same as \c createChannel(request, -1)
+ */
+PendingChannel *ConnectionLowlevel::createChannel(const QVariantMap &request)
+{
+    return createChannel(request, -1);
+}
+
 /**
  * Asynchronously creates a channel satisfying the given request.
  *
- * In typical usage, only the Channel Dispatcher should call this. Ordinary applications should use
- * the Account::createChannel() family of methods (which invoke the Channel Dispatcher's services).
+ * In typical usage, only the Channel Dispatcher should call this. Ordinary
+ * applications should use the Account::createChannel() family of methods
+ * (which invoke the Channel Dispatcher's services).
  *
  * The request MUST contain the following keys:
  *   org.freedesktop.Telepathy.Channel.ChannelType
@@ -1726,20 +1799,17 @@ Client::ConnectionInterface *Connection::baseInterface() const
  * notification of the request finishing processing. See the documentation
  * for that class for more info.
  *
- * The returned PendingChannel object should be freed using
- * its QObject::deleteLater() method after it is no longer used. However,
- * all PendingChannel objects resulting from requests to a particular
- * Connection will be freed when the Connection itself is freed. Conversely,
- * this means that the PendingChannel object should not be used after the
- * Connection is destroyed.
- *
- * \sa PendingChannel
- *
  * \param request A dictionary containing the desirable properties.
+ * \param timeout The D-Bus timeout in milliseconds used for the method call.
+ *                If timeout is -1, a default implementation-defined value that
+ *                is suitable for inter-process communications (generally,
+ *                25 seconds) will be used.
  * \return Pointer to a newly constructed PendingChannel object, tracking
  *         the progress of the request.
+ * \sa PendingChannel
  */
-PendingChannel *ConnectionLowlevel::createChannel(const QVariantMap &request)
+PendingChannel *ConnectionLowlevel::createChannel(const QVariantMap &request,
+        int timeout)
 {
     if (!isValid()) {
         return new PendingChannel(ConnectionPtr(),
@@ -1770,16 +1840,24 @@ PendingChannel *ConnectionLowlevel::createChannel(const QVariantMap &request)
     }
 
     debug() << "Creating a Channel";
-    PendingChannel *channel =
-        new PendingChannel(conn, request, true);
+    PendingChannel *channel = new PendingChannel(conn, request, true, timeout);
     return channel;
 }
 
+/**
+ * Same as \c ensureChannel(request, -1)
+ */
+PendingChannel *ConnectionLowlevel::ensureChannel(const QVariantMap &request)
+{
+    return ensureChannel(request, -1);
+}
+
 /**
  * Asynchronously ensures a channel exists satisfying the given request.
  *
- * In typical usage, only the Channel Dispatcher should call this. Ordinary applications should use
- * the Account::ensureChannel() family of methods (which invoke the Channel Dispatcher's services).
+ * In typical usage, only the Channel Dispatcher should call this. Ordinary
+ * applications should use the Account::ensureChannel() family of methods
+ * (which invoke the Channel Dispatcher's services).
  *
  * The request MUST contain the following keys:
  *   org.freedesktop.Telepathy.Channel.ChannelType
@@ -1791,20 +1869,17 @@ PendingChannel *ConnectionLowlevel::createChannel(const QVariantMap &request)
  * notification of the request finishing processing. See the documentation
  * for that class for more info.
  *
- * The returned PendingChannel object should be freed using
- * its QObject::deleteLater() method after it is no longer used. However,
- * all PendingChannel objects resulting from requests to a particular
- * Connection will be freed when the Connection itself is freed. Conversely,
- * this means that the PendingChannel object should not be used after the
- * Connection is destroyed.
- *
- * \sa PendingChannel
- *
  * \param request A dictionary containing the desirable properties.
+ * \param timeout The D-Bus timeout in milliseconds used for the method call.
+ *                If timeout is -1, a default implementation-defined value that
+ *                is suitable for inter-process communications (generally,
+ *                25 seconds) will be used.
  * \return Pointer to a newly constructed PendingChannel object, tracking
  *         the progress of the request.
+ * \sa PendingChannel
  */
-PendingChannel *ConnectionLowlevel::ensureChannel(const QVariantMap &request)
+PendingChannel *ConnectionLowlevel::ensureChannel(const QVariantMap &request,
+        int timeout)
 {
     if (!isValid()) {
         return new PendingChannel(ConnectionPtr(),
@@ -1835,8 +1910,7 @@ PendingChannel *ConnectionLowlevel::ensureChannel(const QVariantMap &request)
     }
 
     debug() << "Creating a Channel";
-    PendingChannel *channel =
-        new PendingChannel(conn, request, false);
+    PendingChannel *channel = new PendingChannel(conn, request, false, timeout);
     return channel;
 }
 
@@ -2298,12 +2372,36 @@ void Connection::handleRequestLanded(HandleType handleType)
 
 void Connection::onSelfHandleChanged(uint handle)
 {
+    if (mPriv->selfHandle == handle) {
+        return;
+    }
+
+    if (mPriv->pendingStatus != ConnectionStatusConnected || !mPriv->selfHandle) {
+        debug() << "Got a self handle change before we have the initial self handle, ignoring";
+        return;
+    }
+
+    debug() << "Connection self handle changed to" << handle;
     mPriv->selfHandle = handle;
     emit selfHandleChanged(handle);
 
-    if (mPriv->readinessHelper->actualFeatures().contains(FeatureSelfContact)) {
+    if (mPriv->introspectingSelfContact) {
+        // We're currently introspecting the SelfContact feature, but have started building the
+        // contact with the old handle, so we need to do it again with the new handle.
+
+        debug() << "The self contact is being built, will rebuild with the new handle shortly";
+        mPriv->reintrospectSelfContactRequired = true;
+    } else if (isReady(FeatureSelfContact)) {
+        // We've already introspected the SelfContact feature, so we can reinvoke the introspection
+        // logic directly to rebuild with the new handle.
+
+        debug() << "Re-building self contact for handle" << handle;
         Private::introspectSelfContact(mPriv);
     }
+
+    // If ReadinessHelper hasn't started introspecting SelfContact yet for the Connected state, we
+    // don't need to do anything. When it does start the introspection, it will do so using the
+    // correct handle.
 }
 
 void Connection::onBalanceChanged(const Tp::CurrencyAmount &value)
