@@ -306,6 +306,9 @@ bool ManagerFile::Private::parse(const QString &fileName)
                         QLatin1String("allowed"));
 
                 info.rccs.append(rcc);
+
+                rcc.fixedProperties.clear();
+                rcc.allowedProperties.clear();
             }
         }
     }
@@ -363,6 +366,7 @@ QVariant ManagerFile::Private::valueForKey(const QString &param,
 
 /**
  * \class ManagerFile
+ * \ingroup utils
  * \headerfile TelepathyQt4/manager-file.h <TelepathyQt4/ManagerFile>
  *
  * \brief The ManagerFile class provides an easy way to read telepathy manager
