@@ -25,6 +25,7 @@
 #include "TelepathyQt4/_gen/pending-channel.moc.hpp"
 
 #include "TelepathyQt4/debug-internal.h"
+#include "TelepathyQt4/fake-handler-manager-internal.h"
 #include "TelepathyQt4/request-temporary-handler-internal.h"
 
 #include <TelepathyQt4/Channel>
@@ -139,7 +140,7 @@ PendingChannel::PendingChannel(const ConnectionPtr &connection, const QString &e
  * \param create Whether createChannel or ensureChannel should be called.
  */
 PendingChannel::PendingChannel(const ConnectionPtr &connection,
-        const QVariantMap &request, bool create)
+        const QVariantMap &request, bool create, int timeout)
     : PendingOperation(connection),
       mPriv(new Private)
 {
@@ -155,14 +156,14 @@ PendingChannel::PendingChannel(const ConnectionPtr &connection,
         connection->interface<Client::ConnectionInterfaceRequestsInterface>();
     if (create) {
         QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(
-                requestsInterface->CreateChannel(request), this);
+                requestsInterface->CreateChannel(request, timeout), this);
         connect(watcher,
                 SIGNAL(finished(QDBusPendingCallWatcher*)),
                 SLOT(onConnectionCreateChannelFinished(QDBusPendingCallWatcher*)));
     }
     else {
         QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(
-                requestsInterface->EnsureChannel(request), this);
+                requestsInterface->EnsureChannel(request, timeout), this);
         connect(watcher,
                 SIGNAL(finished(QDBusPendingCallWatcher*)),
                 SLOT(onConnectionEnsureChannelFinished(QDBusPendingCallWatcher*)));
@@ -499,13 +500,21 @@ void PendingChannel::onHandlerChannelReceived(const ChannelPtr &channel)
     mPriv->handle = channel->targetHandle();
     mPriv->immutableProperties = channel->immutableProperties();
     mPriv->channel = channel;
+
+    // register the CR in FakeHandlerManager so that at least one handler per bus stays alive
+    // until all channels requested using R&H gets invalidated/destroyed.
+    // This is important in case Mission Control happens to restart while
+    // the channel is still in use, since it will close each channel it
+    // doesn't find a handler for it.
+    FakeHandlerManager::instance()->registerClientRegistrar(mPriv->cr);
+
     setFinished();
 }
 
 void PendingChannel::onAccountCreateChannelFinished(PendingOperation *op)
 {
     if (isFinished()) {
-        if (!isError()) {
+        if (isError()) {
             warning() << "Creating/ensuring channel finished with a failure after the internal "
                 "handler already got a channel, ignoring";
         }
